/**
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at www.sctp.de which should be used for
 * any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 *
 *
 */
import java.io.*;
import java.net.*;


public class AssociationClass {
    private int associationID;
    private short sctpInstance;
    private InetAddress destination=null;
    private short sctpDestPort;
    private int payloadProtocolId;
    private DatagramSocket udpSocket;
    private short numOfAddresses;   // number of dest addresses
    private short primaryAddress;   // index of primary
    private short inStreams;        // number of inbound streams
    private short outStreams;       // number of outbound streams

    private short assocState;   // 0 = assoc down, 1= assoc up
    private byte[] pathStates;  // 0 = path OK, 1= path down


    public AssociationClass(short ID) {
        sctpInstance = ID;
        associationID = 0;
        sctpDestPort  = 0;
        udpSocket = null;
        numOfAddresses = 0;
        primaryAddress  = 0;
        assocState = 0;
        pathStates = new byte[20];
    }

    public synchronized  void commUpNotification(int assoc_id, short status, int noOfDestinations,
             short noOfInStreams, short noOfOutStreams) {
        short i, nDest;
        Integer nD = new Integer(noOfDestinations);
        nDest = nD.shortValue();
        // TODO : Error handling, if Assoc already up.....
        if (assocState == 1) {
            System.err.println("Got an Comm Up for an Association which is up !");
            return;
        } else if ((associationID != 0) && (assocState == 0)) {
            if (associationID != assoc_id) {
                System.err.println("Got an Comm Up for false Association !");
                System.exit(-1);
            }
        }
        associationID =  assoc_id;
        numOfAddresses =  nDest;
        primaryAddress  = 0;
        inStreams =  noOfInStreams;
        outStreams =   noOfOutStreams;
        for (i=0; i<  numOfAddresses; i++) {
             pathStates[i] = 0;
        }
       	System.out.println("Received COMM UP !!!!!!!!!!!!!");
       	System.out.println(">");

        assocState = 1;
    }

    public synchronized  void nwStatusChangeNotification(int assocID, short destAddress, short newState) {
        System.out.print("Network Status Change for Address ");
        System.out.print(destAddress);
        if (newState == 0) {
            System.out.println(" . New State: Path OK!");
            pathStates[destAddress] = 0;
        } else {
            System.out.println(" . New State: Path DOWN !");
            pathStates[destAddress] = 1;
        }
    }

    public synchronized  void commLostNotification(int assocID, short status) {
        short i;

        System.out.print("COMM LOST for AssocID : ");
        System.out.print(assocID);
        System.out.print(". Status: ");
        System.out.println(status);
        // TODO : Error handling, if Assoc already up.....
        associationID =  0;
        numOfAddresses =  0;
        primaryAddress  = 0;
        inStreams =  0;
        outStreams =  0;
        for (i=0; i<  numOfAddresses; i++) {
             pathStates[i] = 1;
        }
        assocState = 0;
    }

    public synchronized boolean isAssocUp() {
        if (assocState == 1)
            return true;
        else if (assocState == 0)
            return false;
        System.err.println("Error in Value of assocState !");
        return false;
    }

    public synchronized void setMyPrimary(int newPrimary) {
        if (newPrimary < numOfAddresses) {
            Integer val = new Integer(newPrimary);
            primaryAddress = val.shortValue();
        }
    }

    public synchronized int getMyPrimary() {
       Short s = new Short(primaryAddress);
       return s.intValue();
    }


    public synchronized short getInstanceName() {
        return  sctpInstance;
    }

    public synchronized void setInstanceName(short newInstance) {
        sctpInstance = newInstance;
    }

    public synchronized int getPayloadProtocolId() {
        return  payloadProtocolId;
    }

    public synchronized void setPayloadProtocolId(int newPPId) {
         payloadProtocolId = newPPId;
    }

    public synchronized void setAssocId(int aid) {
        associationID = aid;
    }

    public synchronized int getAssocId() {
        return associationID;
    }



    public synchronized void setDestination(InetAddress dest) {
        destination = dest;
    }

    public synchronized InetAddress getDestination() {
        return destination;
    }

    public synchronized void setSctpDestPort(short newPort) {
        sctpDestPort = newPort;
    }

    public synchronized short getSctpDestPort() {
        return sctpDestPort;
    }


    public synchronized DatagramSocket getSctpSocket() {
        return udpSocket;
    }

    public synchronized void setSctpSocket(DatagramSocket ulpSocket) {
        udpSocket = ulpSocket;
    }

};

