/**
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at www.sctp.de which should be used for
 * any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 *
 *
 */

import java.io.*;
import java.net.*;
import java.util.*;

public class ConsoleReaderThread extends Thread {

    DataOutputStream dos;
    ByteArrayOutputStream bos;
	
    protected DatagramSocket ulpSocket = null;
    protected short localPort,peerPort;
    protected AssociationClass myAssoc;
    protected NotificationClass myNotifier;

    protected BufferedReader in = null;
    protected boolean moreText = true;
    protected boolean readFromConsole = false;

    public ConsoleReaderThread(AssociationClass association,
                            NotificationClass notification,
                            short LocalPeerPort,
                            short LocalSrcPort,
                            DatagramSocket udpSocket) throws IOException {
        // super(this);
        ulpSocket = udpSocket;
        localPort =  LocalSrcPort;
        peerPort =   LocalPeerPort;
        myAssoc =  association;
        myNotifier =  notification;
        in = new BufferedReader(new InputStreamReader(System.in));
    }

    public void run() {
        String dString = null;

        while (moreText) {
            try {
                dString = getNextInput();
                if (dString == null) {
                    doShutdown();
                    waitForOK(SockReaderThread.ULP_SHUTDOWN_OK);
                } else {
                    doInput(dString);
                }
            } catch (Exception e)  {
                  System.err.println("run(): Caught Exception: " + e.getMessage());
            }
        }
    }

    public void doInput(String inString) {
        String lowerString = inString.toLowerCase();
        if (lowerString.startsWith("associate")) {
            try {
                doAssociate();
                waitForOK(SockReaderThread.ULP_ASSOCIATE_OK);
            } catch (Exception e) {
                System.err.println("exception when calling doAssociate !"+ e.getMessage());
            }
        } else if (lowerString.startsWith("shutdown")) {

                    doShutdown();
                    waitForOK(SockReaderThread.ULP_SHUTDOWN_OK);

        } else if (lowerString.startsWith("close")) {

                System.err.println("Command not implemented ! ");

        } else if (lowerString.startsWith("setprimary")) {

                System.err.println("Command not implemented ! ");

        } else if (lowerString.startsWith("status")) {

                System.err.println("Command not implemented ! ");

        } else if (lowerString.startsWith("setport")) {

            short destPort = 2905;
            myAssoc.setSctpDestPort(destPort);
            myAssoc.setPayloadProtocolId(1);

        } else if (lowerString.startsWith("setpeer")) {
            try {
                InetAddress address = InetAddress.getByName("132.252.150.216");
                myAssoc.setDestination(address);
            } catch (Exception e) {
                  System.err.println("Caught Exception: " + e.getMessage());
            }

        } else if (lowerString.startsWith("help")) {
            System.out.println("The following commands are available :");
            System.out.println("associate, shutdown, close, setprimary, status,setport, setpeer,getsrtt");
        } else {
            if (myAssoc.isAssocUp()) {
                doSend(inString);
                waitForOK(SockReaderThread.ULP_SEND_OK);
            } else {
                System.err.println("Association is not up ! Type <Associate> first");
            }
        }
    }

    private void  waitForOK(byte mesgId) {
        DatagramPacket packet;
        byte[] rbuf;
        byte rtype = 0;
        int result = 0;
        int assocId;
        DataInputStream dis=null;

        packet = myNotifier.get();
        try {
            rbuf = new byte[packet.getLength()];
            rbuf = packet.getData();
            dis = new DataInputStream(new ByteArrayInputStream(rbuf));
            rtype = dis.readByte();
        } catch (Exception e) {
                  System.err.println("Caught Exception: " + e.getMessage());
        }
        if (rtype != mesgId) {
            System.err.println("Got OK MESSAGE that I did not wait for !!!!!!!!!!!!!!!!!!");
        }

        switch (rtype) {
            case SockReaderThread.ULP_REGISTER_OK :
                System.err.println("Received REGISTER_OK from Peer - should have been caught by main thread !");
                // memcpy(&mesg[1], &instance_name, sizeof(gushort));
                System.exit(-1);
                break;
            case SockReaderThread.ULP_INITIALIZE_OK :
                System.out.println("Received INIT_OK from Peer SCTP instance");
                // not yet implemented
                break;
            case SockReaderThread.ULP_ASSOCIATE_OK :
                try {
                    assocId = dis.readInt(); // memcpy(&(mesg[1]), &assoc_id, sizeof(guint32));
                    System.out.print("Received ASSOC_OK from Peer with AssocId ");
                    System.out.println(assocId);
                    myAssoc.setAssocId(assocId);
                } catch (Exception e) {
                      System.err.println("Caught Exception: " + e.getMessage());
                }
                break;
            case SockReaderThread.ULP_SHUTDOWN_OK :
                try {
                    result = dis.readInt(); // memcpy(&(mesg[1]), &result, sizeof(guint32));
                } catch (Exception e) {
                      System.err.println("Caught Exception: " + e.getMessage());
                }
                System.out.print("Received SHUTDOWN_OK from Peer with result ");
                System.out.println(result);
                break;
            case SockReaderThread.ULP_CLOSE_OK :
                System.out.println("Received CLOSE_OK from Peer SCTP instance");
                break;
            case SockReaderThread.ULP_SEND_OK :
                try {
                    result = dis.readInt();                // memcpy(&(mesg[1]), &result, sizeof(guint32));
                } catch (Exception e) {
                      System.err.println("Caught Exception: " + e.getMessage());
                }
                System.out.print("Received SEND_OK from Peer with result ");
                System.out.println(result);
                break;
            case SockReaderThread.ULP_SET_PRIMARY_OK :
                System.out.println("Received SET_PRIMARY_OK from Peer SCTP instance -- N/I");
                break;
            case SockReaderThread.ULP_RECEIVE_OK :
                System.err.println("Received RECEIVE_OK - should be done by SockReaderThread");
                System.exit(-1);
                break;
            case SockReaderThread.ULP_STATUS_OK :
                System.out.println("Received STATUS_OK from Peer SCTP instance");
                break;
            case SockReaderThread.ULP_CHANGE_HB_OK :
                System.out.println("Received SEND_OK from Peer SCTP instance");
                break;
            case SockReaderThread.ULP_REQ_HB_OK :
                System.out.println("Received REQ_HB_OK from Peer SCTP instance");
                break;
            case SockReaderThread.ULP_GET_SRTT_OK :
		        System.out.println("Received GET_SRTT_OK from Peer SCTP instance");
                break;
            default :
        		System.out.print("Received Message of Unknown Type :");
        		System.out.println(rtype);
                break;
	    }


    }

    public void doShutdown() {
        byte[] rbuf = new byte[100];	
		byte rtype;
		bos = new ByteArrayOutputStream(100);
		dos = new DataOutputStream(bos);
       	try {
    		dos.writeByte(SockReaderThread.ULP_SHUTDOWN);
	    	dos.writeShort(myAssoc.getAssocId());  /* the assoc id to shut down */
		    byte[] sbuf = bos.toByteArray();
        	DatagramPacket packet = new DatagramPacket(sbuf,sbuf.length,InetAddress.getByName("127.0.0.1"),peerPort);
            ulpSocket.send(packet);	
        } catch (Exception e) {
            System.err.println("Caught Exception: " + e.getMessage());
            System.exit(1);
        }
      	System.out.println("doShutdown : sending done !");
    }


    public void doAssociate() throws UnknownHostException,IOException {
        byte[] rbuf = new byte[100];	
		byte rtype;
		bos = new ByteArrayOutputStream(100);
		dos = new DataOutputStream(bos);

		dos.writeByte(SockReaderThread.ULP_ASSOCIATE);
		dos.writeShort(myAssoc.getInstanceName());  /* the instance name */
		dos.writeShort(15);  /* the number of output streams */
		dos.writeShort(myAssoc.getSctpDestPort());	/* SCTP destination port */
		dos.writeShort(0);	/* use local port flag 0=false, 1= true */

		dos.writeShort(512);   /* AF type of an address = AF_INET */
		// dos.writeShort(10);   /* AF type of an address = AF_INET6 */
		dos.writeShort(0);   	/* struct s_addr port, which is 0 by definition */
		dos.writeByte(myAssoc.getDestination().getAddress()[0]);
		dos.writeByte(myAssoc.getDestination().getAddress()[1]);
		dos.writeByte(myAssoc.getDestination().getAddress()[2]);
		dos.writeByte(myAssoc.getDestination().getAddress()[3]);

		byte[] sbuf = bos.toByteArray();
    	DatagramPacket packet = new DatagramPacket(sbuf,sbuf.length,InetAddress.getByName("127.0.0.1"),peerPort);
       	try {
            ulpSocket.send(packet);	
        } catch (IOException e) {
            System.err.println("i/o exception when sending user Data !");
            System.exit(1);
        }
      	System.out.println("doAssociate : sending done !");
	}


    public void doSend(String inString) {
        byte[] buf = new byte[1500];
        byte[] sbuf = new byte[1500];
        DatagramPacket packet = null;

    	bos = new ByteArrayOutputStream(1500);
        dos = new DataOutputStream(bos);

      	System.out.print("doSend : ");
      	System.out.println(inString);
			
        if (inString == null) {
            moreText = false;
            // log an error ?!
            return;
        } else {
           	try {
    	        buf = inString.getBytes();
                dos.writeByte(SockReaderThread.ULP_SEND);
                dos.writeInt(myAssoc.getAssocId());  /* the assoc id */
                dos.writeInt(buf.length); /* the user data length */
                dos.writeInt(myAssoc.getPayloadProtocolId());
                dos.writeShort(0); /* the stream id */
                dos.writeShort(myAssoc.getMyPrimary()); /* the dest address index */
                dos.writeInt(0);        /* the context */
                dos.writeShort(0);      /* unordered 0=false, 1= true */
                dos.write(buf, 0, buf.length);
            } catch (Exception e) {
                System.err.println("DoSend : Caught Exception 1 : " + e.getMessage());
                e.printStackTrace();
    		    moreText = false;
            }

           	try {
		    	sbuf = bos.toByteArray();
                // construct a send request from that
    			// send the response to the client at "address" and "port"
                packet = new DatagramPacket(sbuf, sbuf.length, InetAddress.getByName("127.0.0.1"), peerPort);
            } catch (Exception e) {
                System.err.println("DoSend : Caught Exception 2 : " + e.getMessage());
                e.printStackTrace();
    		    moreText = false;
            }
           	try {
                ulpSocket.send(packet);
            } catch (Exception e) {
                System.err.println("DoSend : Caught Exception 3: " + e.getMessage());
                e.printStackTrace();
    		    moreText = false;
            }
        }
    }


    protected String getNextInput() {
        String returnValue = null;
        try {
            System.out.print(">");
            returnValue = in.readLine();
            if ((returnValue.equalsIgnoreCase("QUIT"))) {
                in.close();
        		moreText = false;
                returnValue = null;
                // initiate SHUTDOWN afterwards
            }
        } catch (IOException e) {
            returnValue = "IOException occurred in ConsoleReader";
        }
        return returnValue;
    }
}



