/**
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at www.sctp.de which should be used for
 * any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 *
 *
 */
import java.io.*;
import java.net.*;

public class NotificationClass {

    private short instanceID;
    private int associationID;
    private short  destAddressIndex;
    private int context;
    private DatagramPacket packet;
    private boolean okReceived = false;

    public NotificationClass(short ID) {
        instanceID = ID;
        associationID = 0;
        destAddressIndex = 0;
        context = 0;
    }

    public synchronized DatagramPacket get() {
        while (okReceived == false) {
            try {
                // wait for Producer to put value
                wait();
            } catch (InterruptedException e) {
            }
        }
        okReceived = false;
        // notify Producer that value has been retrieved
        notifyAll();
        return packet;
    }

    public synchronized void put(DatagramPacket value) {
        while (okReceived == true) {
            try {
                // wait for Consumer to get value
                wait();
            } catch (InterruptedException e) {
            }
        }
        packet = value;
        okReceived = true;
        notifyAll();
    }
};

