/**
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at www.sctp.de which should be used for
 * any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 *
 *
 */
import java.io.*;
import java.net.*;


public class ULPClass {

    public static final byte ULP_REGISTER=1;
    public static final byte ULP_REGISTER_OK=2;

    public static final short LocalPeerPort = 5555;
    public static final short LocalSrcPort = 4444;

   	static DatagramSocket ulpSocket;


    public static void main(String[] args) throws java.io.IOException {

        short instanceID = 0;

   		try {
            instanceID = registerWithSCTP();
        } catch (Exception e)  {
            System.err.println("Caught Exception: " + e.getMessage());
        }

        if (instanceID == 0) {
          	System.err.println("Did not get valid Instance ID.");
	   	    System.exit(1);
        }

        AssociationClass assoc = new AssociationClass(instanceID);
        NotificationClass notif = new NotificationClass(instanceID);

        SockReaderThread sreader = new SockReaderThread(assoc, notif, LocalPeerPort, LocalSrcPort, ulpSocket);
        ConsoleReaderThread creader = new ConsoleReaderThread(assoc, notif, LocalPeerPort, LocalSrcPort, ulpSocket);

        sreader.start();
        creader.start();
    }

    // Returns the sctp instance name...........
    public static short registerWithSCTP() throws IOException {
		byte[] sbuf = new byte[1];
		byte[] rbuf = new byte[3];
		byte rtype=0;		
        short sctpInstanceName;
        DataInputStream dis=null;

     	System.out.println("Start by creating a new socket !");

   		try {
   			ulpSocket = new DatagramSocket(LocalSrcPort,InetAddress.getByName("127.0.0.1"));
     	} catch (UnknownHostException e) {
          	System.err.println("Don't know about localhost.");
	   	    System.exit(1);
	   	} catch (IOException e) {
	    	System.err.println("Couldn't use connection to localhost.");
	       	System.exit(1);
		}
	
      	System.out.println("Calling ULP_REGISTER !");
		sbuf[0] = ULP_REGISTER;
        try {
    		DatagramPacket packet = new DatagramPacket(sbuf, sbuf.length,
	    			InetAddress.getByName("127.0.0.1"),LocalPeerPort);

          	System.out.println("Sending packet !");

    		ulpSocket.send(packet);	
        } catch (Exception e)  {
            System.err.println("Caught Exception: " + e.getMessage());
        }

      	System.out.println("Waiting for response !");
  		DatagramPacket rpacket = new DatagramPacket(rbuf, rbuf.length);
        ulpSocket.receive(rpacket);

       	dis = new DataInputStream(new ByteArrayInputStream(rbuf));
   		rtype = dis.readByte();

		if (rtype == ULP_REGISTER_OK) {
			System.out.println("Received REGISTER_OK from Peer SCTP instance");
			sctpInstanceName = dis.readShort();
			System.out.println("Received Instance Name "+ Short.toString(sctpInstanceName));
		} else {
			System.out.println("Received Message "+Byte.toString(rtype)+" from peer");
            sctpInstanceName = 0;
		}
        return  sctpInstanceName;
    }

}
