/*
 *  $Id: main.c,v 1.47 2000/11/02 16:17:04 ajung Exp $
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * main.c  - main program module
 *
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef LINUX
#include <unistd.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* #include "adaptation.h"  */
#include "sctp.h"
#include "mini-ulp.h"

#include <arpa/inet.h>          /* for inet_addr() under both SOLARIS/LINUX */
/* #include "timer_list.h"  */



static union sockunion destinAddrs[10];
static union sockunion sourceAddrs[10];
static unsigned short noOfdestinAddrs = 0;
static unsigned short noOfsourceAddrs = 0;

static unsigned short myPort = 0;
static unsigned short remotPort = 0;

static unsigned short doTransmit;
static boolean useLocalPort;
unsigned short localNumberOfInStreams = 15; /* Our default number of streams.
                                               This is forwarded to SCTP with the
                                               sctp_associate primitive. For z-side, SCTP-control
                                               imports this in order to limit number of
                                               in-streams a-side requests. */
static boolean streamRobin = FALSE; /* By default all data are transmitted over one stream */
boolean unordered = FALSE;


/* this one is global, as we want to know, whether to enable ipv6 calls, even if it may be compiled in */
gboolean support_ipv6;
gushort sctp_instance;
unsigned short udpPort = 0;


static unsigned int abortAfter = 0xFFFFFFFF;
static unsigned int shutdownAfter = 0xFFFFFFFF;
int dropflg = 0;                /* tells sender to drop every dropflg-th frame without sending it, imported
                                   by the adaptation layer. */


void getArgs(int argc, char **argv)
{
    int c;
    extern char *optarg;
    extern int optind;
    int i;
    int tflg = 0;               /* transmit data */
    int sflg = 0;               /* source address */
#ifdef HAVE_IPV6
    int sv6flg = 0;
    int dv6flg = 0;
    int result;
#endif

    int dflg = 0;               /* destination address */
    int hflg = 0;               /* print help */
    int lflg = 0;               /* local port */
    int rflg = 0;               /* remote port */
    int uflg = 0;               /* use local port */
    int bflg = 0;               /* heartbeat on */
    int aflg = 0;               /* abort after */
    int nflg = 0;               /* shutdown after */
    int xflg = 0;               /* read all above from a file */
    int cflg = 0;               /* define chunk length (without ULP prot-ID) */
    int mflg = 0;               /* number of streams */
    int oflg = 0;               /* use all streams in round robin manner */
    int vflg = 0;               /* use all streams in round robin manner */
    int jflg = 0;
    int errflg = 0;
    char sources[500];
    char destins[20][100];
#ifdef HAVE_IPV6
    char v6_sources[5000];
    char v6_destins[20][200];
#endif
    char cfgFile[50];
    char *tokens_str;
    char *tokstr;
    int chunkLength = 20;
    unsigned int zlocalPort;
    unsigned int zremotPort;
    unsigned int localudpPort = 0;
    unsigned int HB_time = 0;

    zlocalPort = 0;
    zremotPort = 0;

    support_ipv6 = FALSE;
#ifdef HAVE_IPV6
    while ((c = getopt(argc, argv, "htuovb:s:l:d:r:x:a:n:p:c:m:6:z:j:")) != EOF)
#else
    while ((c = getopt(argc, argv, "htuovb:s:l:d:r:x:a:n:p:c:m:j:")) != EOF)
#endif
    {
        event_logi(VERBOSE, "%c", c);
        switch (c) {
        case 'u':
            if (uflg)
                errflg++;
            else
                uflg++;
            break;
        case 't':
            if (tflg)
                errflg++;
            else
                tflg++;
            break;
        case 'b':
            if (bflg)
                errflg++;
            else {
                bflg++;
                HB_time = atoi(optarg);
            }
            break;
        case 'r':
            if (rflg)
                errflg++;
            else {
                zremotPort = atoi(optarg);
                if (zremotPort > 0xFFFF)
                    errflg++;
                rflg++;
            }
            break;
        case 'l':
            if (lflg)
                errflg++;
            else {
                zlocalPort = atoi(optarg);
                if (zlocalPort > 0xFFFF)
                    errflg++;
                lflg++;
            }
            break;
        case 'j':
            if (jflg)
                errflg++;
            else {
                localudpPort = atoi(optarg);
                if (localudpPort > 0xFFFF)
                    errflg++;
                jflg++;
            }
            break;
        case 's':
            if (sflg)
                errflg++;
            else {
                strcpy(sources, optarg);
                sflg++;
            }
            break;
#ifdef HAVE_IPV6
        case '6':
            if (sv6flg)
                errflg++;
            else {
                strcpy(v6_sources, optarg);
                sv6flg++;
            }
            break;
        case 'z':
            if (dv6flg > 4) {
                errflg++;
                break;
            }
            strcpy(v6_destins[dv6flg], optarg);
            dv6flg++;
            break;
#endif
        case 'd':
            if (dflg > 4) {
                errflg++;
                break;
            }
            strcpy(destins[dflg], optarg);
            dflg++;
            break;
        case 'a':
            if (aflg)
                errflg++;
            else {
                aflg++;
                abortAfter = atoi(optarg);
            }
            break;
        case 'n':
            if (nflg)
                errflg++;
            else {
                nflg++;
                shutdownAfter = atoi(optarg);
            }
            break;
        case 'p':
            if (dropflg)
                errflg++;
            else {
                dropflg = atoi(optarg);
            }
            break;
        case 'x':
            if (xflg)
                errflg++;
            else {
                xflg++;
                strcpy(cfgFile, optarg);
            }
            break;
        case 'c':
            if (cflg)
                errflg++;
            else {
                cflg++;
                chunkLength = atoi(optarg);
            }
            break;
        case 'm':
            if (mflg)
                errflg++;
            else {
                mflg++;
                localNumberOfInStreams = atoi(optarg);
            }
            break;
        case 'o':
            if (oflg)
                errflg++;
            else {
                oflg++;
            }
            break;
        case 'v':
            if (vflg)
                errflg++;
            else {
                vflg++;
            }
            break;
        case 'h':
            hflg++;
            break;
        case '?':
            errflg++;
        }
    }

    event_log(VERBOSE, "-----------\n");

    if (hflg) {
        printf("usage: sctp [-htu] -s sourceaddr1,sourceaddr2,... -l localSCTPPort     \\ \n");
        printf("            [-d destaddr1 -d destaddr2 ....-r remotePort] [-p n]    \\ \n");
        printf("            [-b HBinterval] [-a #of datachunks] [-n #of datachunks]\n");
#ifdef HAVE_IPV6
        printf("            [-6 ipv6-sourceaddr1,...] [-z ipv6-destaddr1 -d ipv6-destaddr2 ...]\n");
        printf("use -6 option to define source address(es) as IPv6 numerical (hex) addresses.\n");
        printf
            ("use -z option to define destination address(es) as IPv6 numerical (hex) addresses.\n");
#endif
        printf("use -s option to define the source address(es).\n");
        printf("use -l option to define the local port where inits are accepted.\n");
        printf("use -d option to define the destination address.\n");
        printf("use -r option to define the remote port where init is sent to.\n");
        printf("use -t option to activate data transmission\n");
        printf("use -u option to use the local listening port as source port in init\n");
        printf("use -b 'HBinterval' to switch on heartbeat every (RTO+HB.interval) msecs\n");
        printf("use -p n option to drop every n-th frame without sending it (for testing) \n");
        printf("use -a '#of datachunks' to abort after transmission of '#of datachunks'\n");
        printf("use -n '#of datachunks' to shutdown after transmission of '#of datachunks'\n");
        printf("use -c '#of bytes of userdata' to modify chunklength, default is 20 bytes\n");
        printf("use -m '#of streams' enter number of streams, default is 15 streams\n");
        printf("use -o 'to use all streams in round robin manner\n");
        printf("use -v 'to do also unordered transmission\n");
        printf
            ("use -j 'portnum'  to use asynchronous communication via UDP port 'portnum' to ULP\nThis is experimental !\n");

        printf("for multihoming multiple source addresses can be entered after the -s option.\n");
        printf("They must be separated by commas.\n");
        printf
            ("To establish more than one association, the -d option can be entered n-times, each\n");
        printf("followed by a destination address to which the init is sent.\n");
        printf("-d can be ommitted to start a sctp that listens only for incoming inits\n");
        printf("   if -d is present -r is mantadory\n");
        exit(0);
    }


    if (errflg || optind < argc || ((
#ifdef HAVE_IPV6
                                        ((dflg || dv6flg) && !rflg) ||
#else
                                        (dflg && !rflg) ||
#endif
                                        (uflg && !lflg) ||
                                        ((aflg || nflg) && !tflg) || (!dflg && !lflg) ||
#ifdef HAVE_IPV6
                                        (!sflg && !sv6flg)
#else
                                        (!sflg)
#endif
                                    ))) {
        printf("usage: sctp [-hu] -s sourceaddr1,sourceaddr2,... -l localSCTPPort      \\ \n");
        printf("            [-d destaddr1 -d destaddr2 ....-r remotePort]           \\ \n");
        printf("            [-b HBinterval] [-a #of datachunks] [-n #of datachunks]\n");
#ifdef HAVE_IPV6
        printf("            [-6 ipv6-sourceaddr1,...] [-z ipv6-destaddr1 -d ipv6-destaddr2 ...]\n");
#endif
        printf("use sctp -h for help\n");
        exit(2);
    }


    if (zlocalPort == 0 && !dflg) {
        printf("Local port equal to zero is allowed only for clients\n");
        exit(2);
    }

    if (tflg) {
        event_log(VERBOSE, "transmitt data is activated\n");
        ulp_setChunkLength(chunkLength);
        doTransmit = 1;
    } else {
        event_log(VERBOSE, "transmitt data is deactivated\n");
        doTransmit = 0;
    }

    if (uflg)
        useLocalPort = TRUE;
    else
        useLocalPort = FALSE;

    if (vflg) {
        unordered = TRUE;
        event_log(VERBOSE, "some chunks are transmitted unordered\n");
    }

    noOfsourceAddrs = 0;

    if (sflg) {
        event_logi(VERBOSE, "sourceaddresses: %s\n", sources);
        tokens_str = sources;

        while ((tokstr = strtok(tokens_str, ",")) != NULL) {
            tokens_str = NULL;
            event_logi(VERBOSE, "sourceaddresses separated: %s\n", tokstr);
            sourceAddrs[noOfsourceAddrs].sa.sa_family = AF_INET;
            sourceAddrs[noOfsourceAddrs].sin.sin_port = 0;
            sourceAddrs[noOfsourceAddrs].sin.sin_addr.s_addr = inet_addr(tokstr);
            noOfsourceAddrs++;
        }
    }
#ifdef HAVE_IPV6
    if (sv6flg) {
        event_logi(VERBOSE, "IPv6 sourceaddresses: %s\n", v6_sources);
        tokens_str = v6_sources;

        while ((tokstr = strtok(tokens_str, ",")) != NULL) {
            tokens_str = NULL;
            event_logi(VERBOSE, "IPv6 sourceaddresses separated: %s\n", tokstr);
            sourceAddrs[noOfsourceAddrs].sa.sa_family = AF_INET6;
            /* Maybe also assign flow-label, scope id ????????? */
            sourceAddrs[noOfsourceAddrs].sin6.sin6_port = 0;
            result =
                inet_pton(AF_INET6, tokstr, sourceAddrs[noOfsourceAddrs].sin6.sin6_addr.s6_addr);
            if (result != 1)
                error_logi(ERROR_FATAL, "IPv6 Src. Address Conversion returned %d !", result);
            noOfsourceAddrs++;
            support_ipv6 = TRUE;
        }
    }
#endif

    myPort = zlocalPort;
    event_logi(VERBOSE, "Local Port = %hd\n", myPort);

    for (i = 0; i < dflg; i++) {
        event_logii(VERBOSE, "Destinationaddress to Host%d: %s\n", i, destins[i]);
        destinAddrs[noOfdestinAddrs].sa.sa_family = AF_INET;
        destinAddrs[noOfdestinAddrs].sin.sin_port = 0;
        destinAddrs[noOfdestinAddrs].sin.sin_addr.s_addr = inet_addr(destins[i]);
        noOfdestinAddrs++;
    }
#ifdef HAVE_IPV6
    for (i = 0; i < dv6flg; i++) {
        event_logii(VERBOSE, "Destinationaddress to Host%d: %s\n", i, v6_destins[i]);
        destinAddrs[noOfdestinAddrs].sa.sa_family = AF_INET6;
        destinAddrs[noOfdestinAddrs].sin6.sin6_port = 0;
        result =
            inet_pton(AF_INET6, &(v6_destins[i][0]),
                      destinAddrs[noOfdestinAddrs].sin6.sin6_addr.s6_addr);
        if (result != 1)
            error_logi(ERROR_FATAL, "IPv6 Dest. Address Conversion returned %d !", result);
        noOfdestinAddrs++;
        support_ipv6 = TRUE;
    }

#endif
    remotPort = zremotPort;
    event_logi(VERBOSE, "Remote Port = %hd\n", remotPort);

    if (bflg) {
        mulp_heartbeat(HB_time);
        event_logi(VERBOSE, "heartbeat on with heartbeat intervall %u msecs!!", HB_time);
    } else {
        event_log(VERBOSE, "heartbeat disabled !!");
    }

    if (oflg)
        streamRobin = TRUE;

    ulp_getEndEvents(shutdownAfter, abortAfter);

    event_logi(VERBOSE, "number of streams = %u\n", localNumberOfInStreams);

    if (streamRobin) {
        mulp_streamRoundRobin();
        event_log(VERBOSE, "all streams are use in round robin manner\n");
    }

    if (localudpPort == 0 && jflg) {
        error_log(ERROR_FATAL, "Local UDP Port 0 is not allowed !");
    }
    udpPort = localudpPort;

    if (doTransmit)
        mulp_dosend();
}




int main(int argc, char **argv)
{
    int result;
    struct ULPcallback my_ulp, udp_ulp;
    int i;

    read_tracelevels();

    my_ulp.dataArriveNotif = &ulp_dataArriveNotif;
    my_ulp.networkStatusChangeNotif = &ulp_networkStatusChangeNotif;
    my_ulp.sentFailureNotif = &ulp_sentFailureNotif;
    my_ulp.communicationLostNotif = &ulp_communicationLostNotif;
    my_ulp.communicationUpNotif = &ulp_communicationUpNotif;

    udp_ulp.dataArriveNotif = &ulp_udp_dataArriveNotif;
    udp_ulp.networkStatusChangeNotif = &ulp_udp_networkStatusChangeNotif;
    udp_ulp.sentFailureNotif = &ulp_udp_sentFailureNotif;
    udp_ulp.communicationLostNotif = &ulp_udp_communicationLostNotif;
    udp_ulp.communicationUpNotif = &ulp_udp_communicationUpNotif;

    getArgs(argc, argv);


    if (udpPort == 0) {

        if (noOfsourceAddrs > 0) {
            sctp_instance =
                sctp_registerSCTP_instance(FALSE, myPort, noOfsourceAddrs, sourceAddrs, my_ulp);
            event_logi(INTERNAL_EVENT_1, "SCTP-initialized: Instance name : %d\n", sctp_instance);
        } else {
            sctp_instance = sctp_registerSCTP_instance(FALSE, myPort, 0, NULL, my_ulp);
            event_logi(INTERNAL_EVENT_1,
                       "SCTP-initialized without sourceaddresses: Instance name : %d\n",
                       sctp_instance);

        }
    } else {
        if (noOfsourceAddrs > 0) {
            event_logi(INTERNAL_EVENT_1, "Calling with UDP port requested to %u", udpPort);
            sctp_instance =
                sctp_registerSCTP_instance(TRUE, myPort, noOfsourceAddrs, sourceAddrs, udp_ulp);
            event_logi(INTERNAL_EVENT_1,
                       "SCTP-initialized with UDP: Instance name : %d\n", sctp_instance);
        } else {
            sctp_instance = sctp_registerSCTP_instance(TRUE, myPort, 0, NULL, udp_ulp);
            event_logi(INTERNAL_EVENT_1,
                       "SCTP-initialized without sourceaddresses, with UDP : Instance name : %d\n",
                       sctp_instance);

        }

    }

    if (udpPort == 0) {         /* then we start the association, if we are told so */

        if (noOfdestinAddrs > 0) {
            for (i = 0; i < noOfdestinAddrs; i++) {
                event_logi(VERBOSE, "Destination: %08x ", sock2ip(&destinAddrs[i]));
                event_log(INTERNAL_EVENT_0, "Calling sctp_associate() \n");
                result =
                    sctp_associate(sctp_instance, localNumberOfInStreams,
                                   &destinAddrs[i], remotPort, useLocalPort);
            }

        }
    }

    event_log(INTERNAL_EVENT_0, "Main : Entering Event Loop !\n");

    if (noOfdestinAddrs == 0) {
    	printf(">");
	    fflush(stdout);
    }

    while (adl_event_loop() >= 0);

    return EXIT_SUCCESS;
}
