/*
 *  $Id: mini-ulp.h,v 1.23 2000/11/02 16:17:04 ajung Exp $
 *
 * SCTP implementation according to RFC 2960.
 * Copyright (C) 2000 by Siemens AG, Munich, Germany.
 *
 * Realized in co-operation between Siemens AG
 * and University of Essen, Institute of Computer Networking Technology.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * There are two mailinglists available at http://www.sctp.de which should be
 * used for any discussion related to this implementation.
 *
 * Contact: discussion@sctp.de
 *          Michael.Tuexen@icn.siemens.de
 *          ajung@exp-math.uni-essen.de
 *
 * Purpose: Implements a few simple callback-functions for an "ULP"
 */

#ifndef MINI_ULP_H
#define MINI_ULP_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "globals.h"

void ulp_getEndEvents(unsigned int shutdownAfter, unsigned int abortAfter);
void mulp_dosend();
void mulp_heartbeat(unsigned int interval);
void ulp_setChunkLength(int chunkLength);
void mulp_streamRoundRobin();

/* function to handle typed user commands */
/* FOR THE BAKEOFF */
void
ulp_stdin_cb(Socket_descriptor sin_fd, unsigned char *buffer,
             int bufferLength, union sockunion *fromAddress, int fromAddressLength);
void
ulp_socket_error(Socket_descriptor sin_fd,
                 unsigned char *buffer,
                 int bufferLength, union sockunion *fromAddress, int fromAddressLength);


/*  indicates new data have arrived from peer (chapter 9.2.A).
      params: 1. associationID
              2. streamID
*/
void ulp_dataArriveNotif(unsigned int assoc_id, unsigned int stream_id, unsigned int len, unsigned int protoID);
void ulp_udp_dataArriveNotif(unsigned int assoc_id, unsigned int stream_id, unsigned int len, unsigned int protoID);

/* indicates a change of network status (chapter 9.2.C).
   params: 1.  associationID
           2.  destinationAddresses
	       3.  newState
*/
void ulp_networkStatusChangeNotif(unsigned int assoc_id, short dest_add, unsigned short new_state);
void ulp_udp_networkStatusChangeNotif(unsigned int assoc_id, short dest_add,
                                      unsigned short new_state);

/* indicates a sned failure (chapter 9.2.B).
   params: 1.  associationID
           2.  pointer to data not sent
           3.  dataLength
           4.  context from sendChunk
*/
void ulp_sentFailureNotif(unsigned int assoc_id,
                          unsigned char *unsent_data, unsigned int data_len, unsigned int *context);

void ulp_udp_sentFailureNotif(unsigned int assoc_id,
                              unsigned char *unsent_data,
                              unsigned int data_len, unsigned int *context);


/* indicates that communication was lost to peer (chapter 9.2.E).
   params: 1.  associationID
           2.  status, type of event
*/
void ulp_communicationLostNotif(unsigned int assoc_id, unsigned short status);
void ulp_udp_communicationLostNotif(unsigned int assoc_id, unsigned short status);

/* indicates that a association is established (chapter 9.2.D).
   params: 1.  associationID
           2.  status, type of event
*/
void ulp_communicationUpNotif(unsigned int assoc_id, unsigned short status,
                              union sockunion *destinations,
                              int noOfDestinations,
                              unsigned short noOfInStreams, unsigned short noOfOutStreams);

void ulp_udp_communicationUpNotif(unsigned int assoc_id,
                                  unsigned short status,
                                  union sockunion *destinations,
                                  int noOfDestinations,
                                  unsigned short noOfInStreams, unsigned short noOfOutStreams);

#endif
